﻿using System;
using MonoTouch.UIKit;
using MonoTouch.Foundation;
using MonoTouch.CoreGraphics;
using MonoTouch.ObjCRuntime;

namespace NucliOSHomeAnimation_CS
{
	public class Bubble : UIControl
	{
		[Export("Label")]
		public UILabel ContentLabel { get; set; }

		[Export("Fill")]
		public UIImageView Fill { get; set; }
		[Export("Features")]
		public NSArray Features { get; set; }
		[Export("Numbers")]
		public NSArray Numbers { get; set; }
		[Export("FeatureLabel")]
		public UILabel FeatureLabel { get; set; }
		[Export("ControlNameLabel")]
		public UILabel ControlNameLabel { get; set; }
		[Export("Index")]
		public int Index { get; set; }

		UIImageView _outer, _middle, _inner, _otherFill;
		NSTimer _timer;
		float _outerAngle, _middleAngle, _innerAngle;
		UIColor _textColor;
		String _className;

		public Bubble (bool IsRed, string Value, float FontSize, String className)
		{
			string name = (IsRed) ? "RedOrange" : "BlueGreenSlice";

			UIColor color = (IsRed) ? UIColor.FromRGBA(241.0f / 255.0f, 108.0f / 255.0f, 39.0f / 255.0f, 1) : UIColor.FromRGBA(13.0f / 255.0f, 87.0f / 255.0f, 134.0f / 255.0f, 1);

			this.BackgroundColor = UIColor.Clear;

			if (className != null)
				this.AddTarget (this, new Selector ("Navigate"), UIControlEvent.TouchUpInside);

			_className = className;

			_outer = new UIImageView (UIImage.FromBundle (String.Format("Images/{0}-outer.png", name)));
			_inner = new UIImageView (UIImage.FromBundle (String.Format("Images/{0}-inner.png", name)));
			_middle = new UIImageView (UIImage.FromBundle (String.Format("Images/{0}-middle.png", name)));
			Fill = new UIImageView (UIImage.FromBundle (String.Format("Images/{0}-fill.png", name)));
			_otherFill = new UIImageView (UIImage.FromBundle (String.Format("Images/{0}-fill.png", name)));

			this.AddSubview(_outer);
			this.AddSubview(_inner);
			this.AddSubview(_middle);
			this.AddSubview(Fill);
			this.AddSubview(_otherFill);

			Fill.Alpha = 0;
			_otherFill.Alpha = 0;
			_outerAngle = _innerAngle = _middleAngle = 0;
			ContentLabel = new UILabel();
			ContentLabel.BackgroundColor = UIColor.Clear;
			ContentLabel.TextAlignment = UITextAlignment.Center;
			ContentLabel.Text = Value;
			ContentLabel.TextColor = _textColor = color;
			ContentLabel.Font = UIFont.FromName("HelveticaNeue", FontSize);

			this.AddSubview(ContentLabel);
		}

		public override System.Drawing.RectangleF Frame {
			get 
			{
				return base.Frame;
			}
			set 
			{
				if (base.Frame.Equals (value))
					return;
				base.Frame = value;
				_otherFill.Center = Fill.Center = ContentLabel.Center = _outer.Center = _inner.Center = _middle.Center = new System.Drawing.PointF (value.Size.Width / 2, value.Size.Height / 2);
				_otherFill.Bounds = Fill.Bounds = ContentLabel.Bounds = _outer.Bounds = _inner.Bounds = _middle.Bounds = new System.Drawing.RectangleF (0, 0, value.Size.Width, value.Size.Height);
			}
		}

		[Export("Navigate")]
		public void Navigate()
		{
			Type t = Type.GetType (_className);
			UIViewController vc = (UIViewController)Activator.CreateInstance(t);
			UIApplication app = UIApplication.SharedApplication;
			UIWindow window = (UIWindow)app.Windows.GetValue (0);
			UINavigationController nav = (UINavigationController)window.RootViewController;
			nav.PushViewController (vc, true);
		}

		[Export("Pause")]
		public void Pause()
		{
			if (_timer != null)
				_timer.Invalidate ();
			_timer = null;
		}

		[Export("Resume")]
		public void Resume()
		{
			Pause ();
			SetupNextRotate ();
		}

		[Export("Rotate")]
		public void Rotate()
		{
			_outerAngle = RotateView (_outer, _outerAngle, true);
			_middleAngle = RotateView (_middle, _middleAngle, false);
			_innerAngle = RotateView (_inner, _innerAngle, true);
		}

		public void SetupNextRotate()
		{
			int nextRotate = 0;

			while (nextRotate <= 3)
				nextRotate = new Random (DateTime.Now.Millisecond).Next (20);

			_timer = NSTimer.CreateScheduledTimer (nextRotate, this, new Selector("RotateTick"), null, false);
		}

		[Export("RotateTick")]
		public void RotateTick()
		{
			this.SendActionForControlEvents(UIControlEvent.ValueChanged);
			Rotate();
			SetupNextRotate();
		}

		private float RotateView(UIView View, float CurrentAngle, bool Direction)
		{
			if(Direction)
			{
				if(CurrentAngle == 360)
					CurrentAngle = 0;

				CurrentAngle += 45;

			}
			else
			{
				if(CurrentAngle == 0)
					CurrentAngle = 360;

				CurrentAngle -= 45;
			}

			float radians = CurrentAngle * (float)Math.PI / 180;
			UIView.Animate (0.75, () => {
				View.Transform = CGAffineTransform.MakeRotation(radians);
			});

			return CurrentAngle;
		}
	}
}

